@@ -1,4 +1,4 @@
-# Copyright 2011 Google Inc.
+# Copyright 2011 The Kyua Authors.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,7 @@
 
 utils_test_case one_test_program__all_pass
 one_test_program__all_pass_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -54,7 +54,7 @@ EOF
 
 utils_test_case one_test_program__some_fail
 one_test_program__some_fail_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -79,7 +79,7 @@ EOF
 
 utils_test_case many_test_programs__all_pass
 many_test_programs__all_pass_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -93,11 +93,11 @@ EOF
     cat >expout <<EOF
 first:pass  ->  passed  [S.UUUs]
 first:skip  ->  skipped: The reason for skipping is this  [S.UUUs]
+fourth:main  ->  skipped: Required file '/non-existent/foo' not found  [S.UUUs]
 second:pass  ->  passed  [S.UUUs]
 second:skip  ->  skipped: The reason for skipping is this  [S.UUUs]
 third:pass  ->  passed  [S.UUUs]
 third:skip  ->  skipped: The reason for skipping is this  [S.UUUs]
-fourth:main  ->  skipped: Required file '/non-existent/foo' not found  [S.UUUs]
 
 Results file id is $(utils_results_id)
 Results saved to $(utils_results_file)
@@ -115,7 +115,7 @@ EOF
 
 utils_test_case many_test_programs__some_fail
 many_test_programs__some_fail_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -129,11 +129,11 @@ EOF
     cat >expout <<EOF
 first:fail  ->  failed: This fails on purpose  [S.UUUs]
 first:pass  ->  passed  [S.UUUs]
+fourth:main  ->  failed: Returned non-success exit status 76  [S.UUUs]
 second:fail  ->  failed: This fails on purpose  [S.UUUs]
 second:pass  ->  passed  [S.UUUs]
 third:pass  ->  passed  [S.UUUs]
 third:skip  ->  skipped: The reason for skipping is this  [S.UUUs]
-fourth:main  ->  failed: Returned non-success exit status 76  [S.UUUs]
 
 Results file id is $(utils_results_id)
 Results saved to $(utils_results_file)
@@ -153,7 +153,7 @@ EOF
 
 utils_test_case expect__all_pass
 expect__all_pass_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -161,6 +161,7 @@ test_suite("integration")
 atf_test_program{name="expect_all_pass"}
 EOF
 
+# CHECK_STYLE_DISABLE
     cat >expout <<EOF
 expect_all_pass:die  ->  expected_failure: This is the reason for death  [S.UUUs]
 expect_all_pass:exit  ->  expected_failure: Exiting with correct code  [S.UUUs]
@@ -173,6 +174,7 @@ Results saved to $(utils_results_file)
 
 5/5 passed (0 failed)
 EOF
+# CHECK_STYLE_ENABLE
 
     utils_cp_helper expect_all_pass .
     atf_check -s exit:0 -o file:expout -e empty kyua test
@@ -181,7 +183,7 @@ EOF
 
 utils_test_case expect__some_fail
 expect__some_fail_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -189,6 +191,7 @@ test_suite("integration")
 atf_test_program{name="expect_some_fail"}
 EOF
 
+# CHECK_STYLE_DISABLE
     cat >expout <<EOF
 expect_some_fail:die  ->  failed: Test case was expected to terminate abruptly but it continued execution  [S.UUUs]
 expect_some_fail:exit  ->  failed: Test case expected to exit with code 12 but got code 34  [S.UUUs]
@@ -202,6 +205,7 @@ Results saved to $(utils_results_file)
 
 1/6 passed (5 failed)
 EOF
+# CHECK_STYLE_ENABLE
 
     utils_cp_helper expect_some_fail .
     atf_check -s exit:1 -o file:expout -e empty kyua test
@@ -210,7 +214,7 @@ EOF
 
 utils_test_case premature_exit
 premature_exit_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -218,9 +222,10 @@ test_suite("integration")
 atf_test_program{name="bogus_test_cases"}
 EOF
 
+# CHECK_STYLE_DISABLE
     cat >expout <<EOF
-bogus_test_cases:die  ->  broken: Premature exit; test case received signal 9  [S.UUUs]
-bogus_test_cases:exit  ->  broken: Premature exit; test case exited with code 0  [S.UUUs]
+bogus_test_cases:die  ->  broken: Error: Empty test result or no new line. Test case received signal 9  [S.UUUs]
+bogus_test_cases:exit  ->  broken: Error: Empty test result or no new line. Test case exited with code 0  [S.UUUs]
 bogus_test_cases:pass  ->  passed  [S.UUUs]
 
 Results file id is $(utils_results_id)
@@ -228,6 +233,7 @@ Results saved to $(utils_results_file)
 
 1/3 passed (2 failed)
 EOF
+# CHECK_STYLE_ENABLE
 
     utils_cp_helper bogus_test_cases .
     atf_check -s exit:1 -o file:expout -e empty kyua test
@@ -236,7 +242,7 @@ EOF
 
 utils_test_case no_args
 no_args_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -272,7 +278,7 @@ EOF
 
 utils_test_case one_arg__subdir
 one_arg__subdir_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -288,6 +294,7 @@ atf_test_program{name="simple_all_pass"}
 EOF
     utils_cp_helper simple_all_pass subdir
 
+# CHECK_STYLE_DISABLE
     cat >expout <<EOF
 subdir/simple_all_pass:pass  ->  passed  [S.UUUs]
 subdir/simple_all_pass:skip  ->  skipped: The reason for skipping is this  [S.UUUs]
@@ -297,13 +304,14 @@ Results saved to $(utils_results_file)
 
 2/2 passed (0 failed)
 EOF
+# CHECK_STYLE_ENABLE
     atf_check -s exit:0 -o file:expout -e empty kyua test subdir
 }
 
 
 utils_test_case one_arg__test_case
 one_arg__test_case_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -328,7 +336,7 @@ EOF
 
 utils_test_case one_arg__test_program
 one_arg__test_program_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -368,7 +376,7 @@ EOF
 
 utils_test_case many_args__ok
 many_args__ok_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -387,9 +395,9 @@ EOF
     utils_cp_helper simple_some_fail subdir/second
 
     cat >expout <<EOF
+first:pass  ->  passed  [S.UUUs]
 subdir/second:fail  ->  failed: This fails on purpose  [S.UUUs]
 subdir/second:pass  ->  passed  [S.UUUs]
-first:pass  ->  passed  [S.UUUs]
 
 Results file id is $(utils_results_id)
 Results saved to $(utils_results_file)
@@ -416,7 +424,7 @@ EOF
 
 utils_test_case many_args__no_match__all
 many_args__no_match__all_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -440,7 +448,7 @@ EOF
 
 utils_test_case many_args__no_match__some
 many_args__no_match__some_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -476,7 +484,7 @@ EOF
 
 utils_test_case args_are_relative
 args_are_relative_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     mkdir root
     cat >root/Kyuafile <<EOF
@@ -516,7 +524,7 @@ EOF
 
 utils_test_case only_load_used_test_programs
 only_load_used_test_programs_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -538,7 +546,7 @@ Results saved to $(utils_results_file)
 EOF
     CREATE_COOKIE="$(pwd)/cookie"; export CREATE_COOKIE
     atf_check -s exit:0 -o file:expout -e empty kyua test first
-    if test -f "${CREATE_COOKIE}"; then
+    if [ -f "${CREATE_COOKIE}" ]; then
         atf_fail "An unmatched test case has been executed, which harms" \
             "performance"
     fi
@@ -549,8 +557,8 @@ utils_test_case config_behavior
 config_behavior_body() {
     cat >"my-config" <<EOF
 syntax(2)
-test_suites.suite1["X-the-variable"] = "value1"
-test_suites.suite2["X-the-variable"] = "override me"
+test_suites.suite1["the-variable"] = "value1"
+test_suites.suite2["the-variable"] = "override me"
 EOF
 
     cat >Kyuafile <<EOF
@@ -564,7 +572,7 @@ EOF
     utils_cp_helper config config3
 
     atf_check -s exit:1 -o save:stdout -e empty \
-        kyua -c my-config -v test_suites.suite2.X-the-variable=value2 test
+        kyua -c my-config -v test_suites.suite2.the-variable=value2 test
     atf_check -s exit:0 -o ignore -e empty \
         grep 'config1:get_variable.*failed' stdout
     atf_check -s exit:0 -o ignore -e empty \
@@ -588,9 +596,51 @@ EOF
 }
 
 
+utils_test_case config_unprivileged_user
+config_unprivileged_user_body() {
+    cat >"my-config" <<EOF
+syntax(2)
+unprivileged_user = "nobody"
+EOF
+    cat >Kyuafile <<EOF
+syntax(2)
+atf_test_program{name="config1", test_suite="suite1"}
+EOF
+    utils_cp_helper config config1
+
+    CONFIG_VAR_FILE="$(pwd)/cookie"; export CONFIG_VAR_FILE
+    if [ -f "${CONFIG_VAR_FILE}" ]; then
+        atf_fail "Cookie file already created; test case list may have gotten" \
+            "a bad configuration"
+    fi
+
+    CONFIG_VAR_NAME="unprivileged-user"; export CONFIG_VAR_NAME
+    atf_check -s exit:1 -o ignore -e ignore kyua -c my-config test config1
+    [ -f "${CONFIG_VAR_FILE}" ] || \
+        atf_fail "Cookie file not created; test case list did not get" \
+            "configuration variables"
+    value="$(cat "${CONFIG_VAR_FILE}")"
+    [ "${value}" = "nobody" ] || \
+        atf_fail "Invalid value (${value}) in cookie file; test case list did" \
+            "not get the correct configuration variables"
+
+    rm "${CONFIG_VAR_FILE}"
+
+    CONFIG_VAR_NAME="unprivileged_user"; export CONFIG_VAR_NAME
+    atf_check -s exit:1 -o ignore -e ignore kyua -c my-config test config1
+    [ -f "${CONFIG_VAR_FILE}" ] || \
+        atf_fail "Cookie file not created; test case list did not get" \
+            "configuration variables"
+    value="$(cat "${CONFIG_VAR_FILE}")"
+    [ "${value}" = "nobody" ] || \
+        atf_fail "Invalid value (${value}) in cookie file; test case list did" \
+            "not get the correct configuration variables"
+}
+
+
 utils_test_case store_contents
 store_contents_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -652,13 +702,29 @@ EOF
     utils_cp_helper config config1
 
     atf_check -s exit:3 -o empty -e match:"Invalid.*--results-file" \
-        kyua test --results-file=""
+        kyua test --results-file=
+}
+
+
+utils_test_case results_file__reuse
+results_file__reuse_body() {
+    utils_install_stable_test_wrapper
+
+    cat >Kyuafile <<EOF
+syntax(2)
+atf_test_program{name="simple_all_pass", test_suite="integration"}
+EOF
+    utils_cp_helper simple_all_pass .
+    atf_check -s exit:0 -o ignore -e empty kyua test -r results.db
+
+    atf_check -s exit:2 -o empty -e match:"results.db already exists" \
+        kyua test --results-file="results.db"
 }
 
 
 utils_test_case build_root_flag
 build_root_flag_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -701,7 +767,7 @@ EOF
 
 utils_test_case kyuafile_flag__no_args
 kyuafile_flag__no_args_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 This file is bogus but it is not loaded.
@@ -730,7 +796,7 @@ EOF
 
 utils_test_case kyuafile_flag__some_args
 kyuafile_flag__some_args_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 This file is bogus but it is not loaded.
@@ -777,8 +843,8 @@ EOF
     utils_cp_helper interrupts .
 
     kyua \
-        -v test_suites.integration.X-body-cookie="$(pwd)/body" \
-        -v test_suites.integration.X-cleanup-cookie="$(pwd)/cleanup" \
+        -v test_suites.integration.body-cookie="$(pwd)/body" \
+        -v test_suites.integration.cleanup-cookie="$(pwd)/cleanup" \
         test >stdout 2>stderr &
     pid=${!}
     echo "Kyua subprocess is PID ${pid}"
@@ -801,6 +867,7 @@ EOF
     [ ${ret} -ne 0 ] || atf_fail 'No error code reported'
 
     [ -f cleanup ] || atf_fail 'Cleanup part not executed after signal'
+    atf_expect_pass
 
     atf_check -s exit:0 -o ignore -e empty grep 'Signal caught' stderr
     atf_check -s exit:0 -o ignore -e empty \
@@ -808,9 +875,30 @@ EOF
 }
 
 
+utils_test_case exclusive_tests
+exclusive_tests_body() {
+    cat >Kyuafile <<EOF
+syntax(2)
+test_suite("integration")
+EOF
+    for i in $(seq 100); do
+        echo 'plain_test_program{name="race", is_exclusive=true}' >>Kyuafile
+    done
+    utils_cp_helper race .
+
+    atf_check \
+        -s exit:0 \
+        -o match:"100/100 passed" \
+        kyua \
+        -v parallelism=20 \
+        -v test_suites.integration.shared_file="$(pwd)/shared_file" \
+        test
+}
+
+
 utils_test_case no_test_program_match
 no_test_program_match_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -833,7 +921,7 @@ EOF
 
 utils_test_case no_test_case_match
 no_test_case_match_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -922,7 +1010,7 @@ EOF
 
 utils_test_case bogus_test_program
 bogus_test_program_body() {
-    utils_install_timestamp_wrapper
+    utils_install_stable_test_wrapper
 
     cat >Kyuafile <<EOF
 syntax(2)
@@ -933,15 +1021,17 @@ EOF
     utils_cp_helper bad_test_program crash_on_list
     echo 'I am not executable' >non_executable
 
+# CHECK_STYLE_DISABLE
     cat >expout <<EOF
-crash_on_list:__test_cases_list__  ->  broken: Tester did not exit cleanly: kyua-atf-tester: Invalid test cases list header 'This is not a valid test program!'  [S.UUUs]
-non_executable:__test_cases_list__  ->  broken: Tester did not exit cleanly: kyua-atf-tester: execvp failed: Permission denied  [S.UUUs]
+crash_on_list:__test_cases_list__  ->  broken: Invalid header for test case list; expecting Content-Type for application/X-atf-tp version 1, got ''  [S.UUUs]
+non_executable:__test_cases_list__  ->  broken: Permission denied to run test program  [S.UUUs]
 
 Results file id is $(utils_results_id)
 Results saved to $(utils_results_file)
 
 0/2 passed (2 failed)
 EOF
+# CHECK_STYLE_ENABLE
     atf_check -s exit:1 -o file:expout -e empty kyua test
 }
 
@@ -961,9 +1051,11 @@ atf_test_program{name="i-am-missing"}
 EOF
     echo 'I should not be touched because the Kyuafile is bogus' >subdir/ok
 
+# CHECK_STYLE_DISABLE
     cat >experr <<EOF
 kyua: E: Load of 'Kyuafile' failed: .*Non-existent test program 'subdir/i-am-missing'.
 EOF
+# CHECK_STYLE_ENABLE
     atf_check -s exit:2 -o empty -e "match:$(cat experr)" kyua list
 }
 
@@ -992,10 +1084,12 @@ atf_init_test_cases() {
     atf_add_test_case only_load_used_test_programs
 
     atf_add_test_case config_behavior
+    atf_add_test_case config_unprivileged_user
 
     atf_add_test_case store_contents
     atf_add_test_case results_file__ok
     atf_add_test_case results_file__fail
+    atf_add_test_case results_file__reuse
 
     atf_add_test_case build_root_flag
 
@@ -1004,6 +1098,8 @@ atf_init_test_cases() {
 
     atf_add_test_case interrupt
 
+    atf_add_test_case exclusive_tests
+
     atf_add_test_case no_test_program_match
     atf_add_test_case no_test_case_match
 
