@@ -34,6 +34,7 @@
 
 #include "model/test_program_fwd.hpp"
 
+#include <memory>
 #include <ostream>
 #include <string>
 #include <vector>
@@ -42,7 +43,6 @@
 #include "model/test_case_fwd.hpp"
 #include "utils/fs/path_fwd.hpp"
 #include "utils/noncopyable.hpp"
-#include "utils/shared_ptr.hpp"
 
 namespace model {
 
