@@ -133,13 +133,10 @@ static const bool have_unmount2 =
 /// This is only provided to allow our code to compile in all platforms
 /// regardless of whether they actually have an unmount(2) or not.
 ///
-/// \param unused_path The mount point to be unmounted.
-/// \param unused_flags The flags to the unmount(2) call.
-///
 /// \return -1 to indicate error, although this should never happen.
 static int
-unmount(const char* UTILS_UNUSED_PARAM(path),
-        const int UTILS_UNUSED_PARAM(flags))
+unmount(const char* /* path */,
+        const int /* flags */)
 {
     PRE(false);
     return -1;
